{% extends BASE_TEMPLATE %}

{% load staticfiles %}

{% block head-title %}{{ title }}{% endblock %}

{% block main %}
<link rel="stylesheet" type="text/css" href="{% static 'bom/style.css' %}" />
{% endblock %}

{% block menu %}
{% endblock%}

{% block content %}
Form goes here..
{% endblock %}